<?php

/* $Id: function.inc.php 3987 2010-09-10 13:07:10Z denis $ */

/**
 * Функция дял генерации хэша
 * @return string хэш
 */
function nc_captcha_generate_hash() {
  return  md5(uniqid(rand(), true));
}

/**
 * Формирование кодя для картинки
 * @param string $captcha_hash соответствующий хэш-код
 * @return string символы для каптчи
 */
function nc_captcha_generate_code($captcha_hash) {
  $nc_core = nc_Core::get_object();
  // БД и настройки модуля
  $db = $nc_core->db;
  $MODULE_VARS = $nc_core->modules->get_module_vars();

  // Настройки параметров алгоритма генерации кода
  $alphabet   = ( $MODULE_VARS['captcha']['ALPHABET'] ? $MODULE_VARS['captcha']['ALPHABET'] : 'ABCDEFGHKLMNOPRSTUVWXYZ');
  $num_chars  = ( $MODULE_VARS['captcha']['NUMBER_OF_CHARS'] ? $MODULE_VARS['captcha']['NUMBER_OF_CHARS'] : 5);

  if (strpos($num_chars, "..")) {
    $num_chars = nc_preg_split("|\.+|", $num_chars);
    $num_chars = rand($num_chars[0], $num_chars[1]);
  }

  // Генерация случайной последовательности символов
  $alphabet_size = strlen($alphabet);
  $captcha_code = '';
  for ($i = 0; $i < $num_chars; $i++) {
    $captcha_code .= $alphabet[rand(0, $alphabet_size - 1)];
  }

  // Сохранить сгенерированный код
  $db->query("REPLACE INTO `Captchas` (`Captcha_Hash`, `Captcha_Code`)
              VALUES ('".$db->escape($captcha_hash)."', '".$db->escape($captcha_code)."')");

   return $captcha_code;
}


/**
 * Проверка кода
 * @param string $user_code символы, введенные с картинки
 * @param string $user_hash соответсвующий ему хэш
 * @param bool $delete_hash удалить хеш после проверки или нет
 * @return bool прошла проверка или нет
 */
function nc_captcha_verify_code($user_code, $user_hash = '', $delete_hash = 1) {
  $nc_core = nc_Core::get_object();
  // БД и настройки модуля
  $db = $nc_core->db;
  $MODULE_VARS = $nc_core->modules->get_module_vars();

  // если код не задан - то он заведомо неверный
  if (!$user_code) {
      return false;
  }
  // имя скрытого поля, через которое передается хэш-код
  $hidden_field_name = ( $MODULE_VARS['captcha']['HIDDEN_FIELD_NAME'] ? $MODULE_VARS['captcha']['HIDDEN_FIELD_NAME'] : 'nc_captcha_hash' );

  // Если хэш не передан, то получить его из GET или POST параметров
  if (!$user_hash || 1) { 
    $user_hash = $nc_core->input->fetch_get_post($hidden_field_name);
  }

  // время жизни каптчи
  $captcha_duration = ( $MODULE_VARS['captcha']['DURATION'] ? $MODULE_VARS['captcha']['DURATION'] : 5 * 60 );

  // проверка каптчи
  $captchas_count = (int) $db->get_var("SELECT COUNT(*)
                                        FROM `Captchas`
                                        WHERE UPPER(`Captcha_Code`) = UPPER('".$db->escape($user_code)."')
                                        AND `Captcha_Hash` = '".$db->escape($user_hash)."'
                                        AND UNIX_TIMESTAMP(NOW()) - UNIX_TIMESTAMP(`Captcha_Created`) <= ".intval($captcha_duration));

  $captcha_valid = ($captchas_count == 1);

  // удаление устарвших значений
  if ( $delete_hash ) {
    $db->query("DELETE FROM `Captchas`
                WHERE `Captcha_Hash` = '".$db->escape($user_hash)."'
                OR UNIX_TIMESTAMP(NOW()) - UNIX_TIMESTAMP(`Captcha_Created`) >= ".intval($captcha_duration));
  }

  return $captcha_valid;
}


/**
 * Функция для вывода формы с картинкой и скрытым полем
 * @param <type> $attributes атрибуты, которые попадут в тэг img
 * @return string html-код
 */
function nc_captcha_formfield($attributes = '') {
  $nc_core = nc_Core::get_object();
  // настройки модуля
  $MODULE_VARS = $nc_core->modules->get_module_vars();

  $hidden_field_name = ( $MODULE_VARS['captcha']['HIDDEN_FIELD_NAME'] ? $MODULE_VARS['captcha']['HIDDEN_FIELD_NAME'] : 'nc_captcha_hash');
  $captcha_hash = nc_captcha_generate_hash();

  return "<input type='hidden' name='".$hidden_field_name."' value='".$captcha_hash."' /><img name='nc_captcha_img' src='".$nc_core->SUB_FOLDER.$nc_core->HTTP_ROOT_PATH."modules/captcha/img.php?code=".$captcha_hash."'".($attributes ? " ".$attributes : "")." />";
}


/**
 * Формирование картинки
 * @param string $captcha_hash хэш
 * @return картинка
 */
function nc_captcha_image($captcha_hash) {
  $nc_core = nc_Core::get_object();
  // настройки модуля
  $MODULE_VARS = $nc_core->modules->get_module_vars();

  $captcha_code = nc_captcha_generate_code($captcha_hash);

  // Если пользователем определена собственная функция для создания картинки,
  // вернуть ее результат
  include_once($nc_core->MODULE_FOLDER."captcha/user_functions.inc.php");
  if (function_exists('nc_captcha_user_image')) { return nc_captcha_user_image($captcha_code); }

  // "Стандартный" алгоритм
  // Функция работает только при наличии библиотеки GD с поддержкой GIF
  if (!function_exists("imagegif")) {
    return false;
  }

  $num_chars = strlen($captcha_code);

  // Настройки параметров алгоритма генерации картинки

  $img_width  = ( $MODULE_VARS['captcha']['IMAGE_WIDTH'] ? $MODULE_VARS['captcha']['IMAGE_WIDTH'] : 150 );
  $img_height = ( $MODULE_VARS['captcha']['IMAGE_HEIGHT'] ? $MODULE_VARS['captcha']['IMAGE_HEIGHT'] : 30 );
  $num_lines  = ( isset($MODULE_VARS['captcha']['NUMBER_OF_LINES']) ? $MODULE_VARS['captcha']['NUMBER_OF_LINES'] : 30 );

  $letter_width = (int) ($img_width / $num_chars);
  $captcha_image = imagecreate($img_width, $img_height);

  // Белый фон
  imagecolorallocate($captcha_image, 255, 255, 255);

  $background_color = imagecolorallocate($captcha_image, 255, 255, 255);
  $grey_color = rand(165, 225);
  $noise_color = imagecolorallocate($captcha_image, $grey_color, $grey_color, $grey_color);

  for( $i=0; $i<$num_lines; $i++ ) {
    imageline($captcha_image, mt_rand(0,$img_width), mt_rand(0,$img_height), mt_rand(0,$img_width), mt_rand(0,$img_height), $noise_color);
  }
  
  for ($i = 0; $i < $num_chars; $i++) {
    $font_size = $img_height * round(rand(50, 75)/100, 2);
    $text_color = imagecolorallocate($captcha_image, rand(0, 128), rand(0, 128), rand(0, 128));
    if ( function_exists("imagettftext") ) {
      $rand_x = rand(0, $letter_width - $font_size);
      $rand_y = $img_height - round( ($img_height - $font_size)/2 );
      imagettftext($captcha_image, $font_size, rand(-30,30), $i * $letter_width + $rand_x, $rand_y, $text_color, $nc_core->MODULE_FOLDER."captcha/font.ttf" , $captcha_code[$i]);
    }
    else {
      $rand_font = rand(2, 5);
      $rand_x = rand(0, $letter_width - imagefontwidth($rand_font));
      $rand_y = rand(0, $img_height - imagefontheight($rand_font));
      imagestring($captcha_image, $rand_font, $i * $letter_width + $rand_x, $rand_y, $captcha_code[$i], $text_color);
    }
  }
  
  if ( preg_match("/^(\d)/is", phpversion(), $matches) && function_exists("imagettftext") && function_exists("imagefilter") ) {
    if ($matches[1]==5) imagefilter($captcha_image, IMG_FILTER_GAUSSIAN_BLUR);
  }
  
  return imagegif($captcha_image);
}


?>