<?php
/* $Id: array_to_form.inc.php 4016 2010-09-24 14:35:52Z denis $ */

/**
  * [Array to Form]
  * Класс для преобразования массива с данными + массива с настройками
  * в элементы html-формы и сохранения результатов формы в виде строки
  */
class nc_a2f {
  /**
    * поля формы
    */
  var $fields = array();
  /**
    * ошибки, возникшие при проверке значений
    */
  var $validation_errors = array();
  /**
    * префикс: использовать у элементов формы в названии элементов
    * (e.g. если prefix='settings', названия полей формы будут
    *  вида name='settings[field1]')
    */
  var $array_name;

  /**
    * конструктор
    * @param mixed массив с настройками или соответствующая строка
    *   E.g. $settings_array = array(
    *                            "field1_name" => array("type" => "string")
    *                          );
    * @param mixed массив со значениями или строка
    *        $values_array = array("field1_name" => 123)
    * @param string
    */
  function nc_a2f($settings, $values, $array_name="nc_a2f") {

    $settings_array = is_array($settings) ? $settings : $this->eval_value($settings);
    $values_array = is_array($values) ? $values : $this->eval_value($values);

    if (!is_array($settings_array)) {
      trigger_error("nc_a2f::nc_a2f() - first parameter is not an array or cannot be evaluated as an array", E_USER_WARNING);
      return;
    }

    foreach ((array)$settings_array as $field_name=>$field_settings) {
      if (!$field_settings['type']) {
        trigger_error("(nc_a2f) missing type for the field '$field_name'", E_USER_WARNING);
        continue;
      }
      $class_name = "nc_a2f_field_{$field_settings['type']}";
      if (!class_exists($class_name)) {
        trigger_error("(nc_a2f) wrong type '$field_settings[type]' for the field '$field_name'", E_USER_WARNING);
        continue;
      }

      // push name into settings array
      $field_settings['name'] = $field_name;
      $this->fields[$field_name] = new $class_name($field_settings, $this);
      if ( isset($values_array[$field_name]) && strlen($values_array[$field_name])) {
        $this->fields[$field_name]->set_value($values_array[$field_name]);
      }
    } // of "each field"
    $this->array_name = $array_name;
  } //  of nc_a2f constructor

  /**
    * Eval PHP-кода в значение
    */
  function eval_value($value_string) {
    if (!$value_string) return $value_string;
    $value_string = nc_preg_replace("/;\s*$/", "", $value_string);
    @eval("\$ret = ($value_string);");
    if (strlen($value_string) > 1 && !is_array($ret)) {
      trigger_error("nc_a2f::eval_value - wrong parameter?<pre>".htmlspecialchars($value_string)."</pre>", E_USER_WARNING);
    }
    return $ret;
  }

  /**
    * Вывод элементов
    * @return string
    */
  function render($header = "", $template = "", $footer = "", $divider = "") {
    $ret = $this->render_header($header ? $header : "");
    
  	foreach ($this->fields as $field) {
  		$ret.= $field->render( $field->type=='divider' && $divider ? $divider : ($template ? $template : "") );
  	}

  	$ret.= $this->render_footer($footer ? $footer : "");
    
    return $ret;
  }

  /**
    * @access private
    */
  function render_header($template = "") {
  	if(!$template) {
  		$ret = "<div class='ncf_container'>\n".
        "<div class='ncf_header_row'>".
        "<div class='ncf_header_caption'>".CONTROL_CLASS_CUSTOM_SETTINGS_PARAMETER."</div>" .
        "<div class='ncf_header_default'>".CONTROL_CLASS_CUSTOM_SETTINGS_DEFAULT."</div>" .
        "<div class='ncf_header_value'>".CONTROL_CLASS_CUSTOM_SETTINGS_VALUE."</div>" .
        "</div>\n";
  	}
  	else {
  		$ret = str_replace(
        array("%CAPTION", "%DEFAULT", "%VALUE"),
        array(CONTROL_CLASS_CUSTOM_SETTINGS_PARAMETER, CONTROL_CLASS_CUSTOM_SETTINGS_DEFAULT, CONTROL_CLASS_CUSTOM_SETTINGS_VALUE),
        $template
      );
  	}
  	
    return $ret;
  }

  function render_footer($template="") {
  	if(!$template) $ret = "</div>";
  	else $ret = $template;
    
    return $ret;
  }

  /**
    * Получить строку со значениями (php $values_array)
    * @return string  'f1'=>'value', 'f2=>'value'...
    */
  function get_values_as_string() {
    $all_values = array();
    foreach ($this->fields as $field_name => $field) {
      $val = $this->fields[$field_name]->get_value(); // don't use default value
      if (isset($val)) {
        $val = str_replace("'", "\\'", $val);
        $all_values[] = "'".$field_name."' => '".$val."'";
      }
    }

    if (sizeof($all_values)) {
      $ret = "$".$this->array_name." = array(".join(', ', $all_values).");";
      return $ret;
    }
    return "";
  }

  /**
    * Получить все значения в виде массива
    * @return array
    */
  function get_values_as_array() {
    $values = array();
    foreach ($this->fields as $field_name => $field) {
      $values[$field_name] = $this->fields[$field_name]->get_value();
    }
    return $values;
  }

  /**
    * принять сообщение об ошибке от поля
    */
  function set_validation_error($field_name, $error_msg) {
    if (!$error_msg) {
      $error_msg = str_replace("%NAME", $this->fields[$field_name]->caption, NETCAT_MODERATION_MSG_TWO);
    }
    $this->validation_errors[$field_name] = $error_msg;
  }

  function get_validation_errors() {
    if (!sizeof($this->validation_errors)) return "";

    $ret = "";
    foreach ($this->validation_errors as $field=>$error) {
      $ret .= $this->fields[$field]->caption. ": $error<br>\n";
    }
    return $ret;
  }

  function get_field_error($field_name) {
    return $this->validation_errors[$field_name];
  }

  /**
    * были ли ошибки при заполнении формы?
    * @return boolean
    */
  function has_errors() {
    return (sizeof($this->validation_errors) ? true : false);
  }

}

// ============================================================================

/**
  * Базовый класс полей a2f
  */
class nc_a2f_field {
  var $name;
  var $value;
  var $default_value = "";
  var $validate_regexp;
  /**
    * родительская форма - чтобы
    */
  var $parent;

  /**
    * @param array
    */
  function nc_a2f_field($field_settings, &$parent) {
    foreach ((array)$field_settings as $k=>$v) {
      $this->$k = $v;
    }
    $this->parent = &$parent;
  }

  function set_value($value) {
    if ($this->validate_value($value)) {
      $this->value = $value;
    }
  }

  /**
    * проверка значения на основе validate_regexp и оповещение
    * родительского объекта (nc_a2f) в случае, если значение не
    * соответствует условиям
    * @param mixed
    * @return boolean passed check
    */
  function validate_value($value) {
    if ($this->validate_regexp && !preg_match($this->validate_regexp, $value)) {
      $this->parent->set_validation_error($this->name, $this->validate_error);
      return false;
    }
    return true;
  }

  function get_value($use_default = true) {
    if (isset($this->value) || !$use_default) { return ($this->value); }
    return $this->default_value;
  }

  function render($template = "") { 
  	if(!$template) {
  	$ret = $this->render_prefix() .
             $this->render_default_value() .
             $this->render_value_field() .
             $this->render_suffix();
  	}
  	else {
  		$ret = str_replace(
        array("%CAPTION", "%DEFAULT", "%VALUE"),
        array( $this->render_prefix(false), $this->render_default_value(false), $this->render_value_field(false) ),
        $template
      );
      $ret.= $this->render_suffix(false);
  	}
  	
    return $ret;
  }

  /**
    * @access private
    */
  function render_prefix($html = true) {
    
  	$err = $this->parent->get_field_error($this->name);

  	if ($html) {
      $ret = "<div class='ncf_row".($err ? " ncf_error":"")."'>".
        "<div class='".( $this->type=='divider' ? 'ncf_divider' : 'ncf_caption' )."'>{$this->caption}</div>";
    }
    else {
      $ret = $this->caption;
    }
  
    return $ret;
  }

  /**
    * реализация - в конкретных классах
    * @access private
    */
  function render_value_field($html = true) {}
  
  /**
    * @access private
    */
  function render_default_value($html = true) {
    
    if($this->type != 'divider') {
      if ($html) {
        $ret = "<div class='ncf_default'>{$this->default_value}</div>";
      }
      else {
        $ret = $this->default_value;
      }
    }
		
    return $ret;
  }
  /**
    * @access private
    */
  function render_suffix($html = true) {
		
    if ($html) $ret = "</div>\n";
	
    return $ret;
  }

  /**
    * @access private
    */
  function get_field_name() {
    $array_name = $this->parent->array_name;
    return ($array_name ? "{$array_name}[{$this->name}]" : $this->name);
  }
}

// ---------------------------------------------------------------------------
class nc_a2f_field_string extends nc_a2f_field {
  /**
    * @access private
    */
  function render_value_field($html = true) {
    
	if (!$this->size) $this->size = 20;
	$ret = "<input name='".$this->get_field_name()."' type='text' value='".htmlspecialchars(($this->value ? $this->value : $this->default_value), ENT_QUOTES)."' size='".$this->size."'  class='ncf_value_text'>";
	if ($html) {
    $ret = "<div class='ncf_value'>".$ret."</div>\n";
  }
  
  return $ret;
  }
}

// ---------------------------------------------------------------------------
class nc_a2f_field_select extends nc_a2f_field {
  /**
    * возможные значения (значение - описание)
    */
  var $values;
  /**
    * @access private
    */
  function render_value_field($html = true) {
    $ret = "<select name='".$this->get_field_name()."'  class='ncf_value_select'>\n";

    // текущее значение
  	$current_value = $this->get_value(0);
  	// значение по-умолчанию
  	if(!$current_value) $current_value = $this->default_value;
  	// если нет значения по-умолчанию - выводим пустую строку тоже
  	if(!$this->default_value) $ret .= "<option value=''></option>\n";

    foreach ((array)$this->values as $value=>$description) {
       $ret .= "<option value='$value'".($value==$current_value ? " selected" : "").">$description</option>\n";
    }

  	if ($html) {
      $ret = "<div class='ncf_value'>".$ret."</select></div>\n";
    }
	
    return $ret;
  }

  /**
    * @access private
    */
  function render_default_value($html = true) {
	
	if ($html) {
    $ret = "<div class='ncf_default'>".$this->values[$this->default_value]."</div>";
  }
  else {
    $ret = $this->values[$this->default_value];
  }
	
  return $ret;
  }

  /**
    * проверка значения - принимаются только те значения, которые есть
    * в $this->values.
    */
  function validate_value($value) {
    if (!$this->values[$value]) {
      $this->parent->set_validation_error($this->name, $this->validate_error);
      return false;
    }
    return true;
  }

}

// ---------------------------------------------------------------------------
class nc_a2f_field_checkbox extends nc_a2f_field {
  /**
    * @access private
    */
  function render_value_field($html = true) {
    
	$ret = "<input name='".$this->get_field_name()."' type='checkbox' ".( $this->value == true ? " checked" : "")."  class='ncf_value_checkbox'>";
  if ($html) {	
    $ret = "<div class='ncf_value'>".$ret."</div>\n";
  }
  
  return $ret;
  }
}

// ---------------------------------------------------------------------------
class nc_a2f_field_textarea extends nc_a2f_field {
  /**
    * @access private
    */
  function render_value_field($html = true) {
    
	if (!$this->size) $this->size = 5;
    $ret = "<textarea name='".$this->get_field_name()."' rows='".$this->size."' class='ncf_value_textarea'>".htmlspecialchars(($this->value ? $this->value : $this->default_value), ENT_QUOTES)."</textarea>";
 	if ($html) {
    $ret = "<div class='ncf_value'>".$ret."</div>\n";
  }
	
  return $ret;
  }
}

// ---------------------------------------------------------------------------
class nc_a2f_field_divider extends nc_a2f_field {
  /**
    * @access private
    */
  function render_value_field($html = true) {    
	
    $ret = '';
	
	return $ret;
  }
}
?>