<?php
/* $Id: nc_subdivision.class.php 4026 2010-09-29 14:55:59Z denis $ */

class nc_Subdivision extends nc_Essence {
  
  protected $db;
  
  /**
   * Constructor function
   */  
  public function __construct () {
    // load parent constructor
    parent::__construct();
    
    // system superior object
    $nc_core = nc_Core::get_object();
    // system db object
    if ( is_object($nc_core->db) ) $this->db = $nc_core->db;
    
  }

  public function get_by_id ($id, $item = "") {

     $res = array();
     if ( isset($this->data[$id])) $res = $this->data[$id];


     if ( empty($res) ) {
       $res = $this->db->get_row("SELECT * FROM `Subdivision` WHERE `Subdivision_ID` = '".intval($id)."'", ARRAY_A );
       if ( empty($res) ) {
         //return false;
         throw new Exception("Subdivision with id  ".$id." does not exist");
       }

       $this->data[$id] = $res;
       $this->data[$id]['_nc_final'] = 0;

     }

     if ( !$this->data[$id]['_nc_final'] ) {
       $this->data[$id] = $this->convert_system_vars($this->data[$id]);
       $this->data[$id] = $this->inherit($this->data[$id]);
       $this->data[$id]['_nc_final'] = 1;
     }

     if ( $item ) {
       return array_key_exists($item, $this->data[$id]) ? $this->data[$id][$item] : "";
     }

     return $this->data[$id];


   }


   public function set_current_by_id ($id, $reset = false) {
     $nc_core = nc_Core::get_object();
     try {
       $this->current = $this->get_by_id($id, "", $reset);
     }
     catch ( Exception $e ) {
       $this->current = $this->get_by_id($nc_core->catalogue->get_current("Title_Sub_ID"));
     }

     // return result
     return $this->current;
    

  }

  /**
   * Get subdivision data by the URI parameter
   *
   * @param URI string
   * @param bool use returned value as current catalogue data
   * @param bool reset stored data in the static variable
   *
   * @return subdivision data associative array
   */
  public function set_current_by_uri ($reset = false) {
    // system superior object
    $nc_core = nc_Core::get_object();
    $catalogue = $nc_core->catalogue->get_current("Catalogue_ID");
   
    // URI
    $uri = $nc_core->url->get_parsed_url('path');

    // validate Hidden_URL
    //if ( !$this->validate_hidden_url($uri) ) return false;

     // find date in url
     nc_preg_match("|
        / (\d{4}) /
        (?:
          (\d{2}) /
        )?
        (?:
          (\d{2}) /
        )?
      |x", $uri, $regs);
      // date found
      if ($regs) {
        // crop date symbols
        $uri = str_replace($regs[0], "/", $uri);
      }
      $res = array();

      if ( $uri == "/" || $uri == "" ) { 
        $res = $this->get_by_id($nc_core->catalogue->get_current("Title_Sub_ID"));
      }

      if ( strrchr($uri, '/') != '/' ) $uri .= '/';

      if ( empty($res) &&  !empty($this->data) ) {
        foreach ( $this->data as $id => $values ) {
          if ( $values['Catalogue_ID'] == $catalogue  && $values['Hidden_URL'] = $uri) {
            $res = $this->data[$id];
          }
        }
      }

      if ( empty($res) ) {
        $res = $this->db->get_row("SELECT * FROM `Subdivision`
                                   WHERE `Catalogue_ID` = '".intval( $catalogue )."'
                                   AND `Hidden_URL` = '".$this->db->escape($uri)."'", ARRAY_A);
        if ( $res) {
          $res['_nc_final'] = 0;
          $this->data[$res['Subdivision_ID']] = $res;
        }
        else {
          $res = $this->get_by_id($nc_core->catalogue->get_current("E404_Sub_ID"));
        }
      }

      // processing system fields, inherit
      if (  !$res['_nc_final'] ) {
        $res = $this->get_by_id( $res['Subdivision_ID']);
      }

      $this->current = $res;
    
      return $res;
  }
  
  public function validate_hidden_url ($url) {
    // validate Hidden_URL
    return nc_preg_match("/^[\/_a-zа-я0-9-]+$/i", $url);
  }
  
  protected function inherit ($sub_env) {
    // system superior object
    $nc_core = nc_Core::get_object();
    
    $sub = $sub_env["Subdivision_ID"];
    
    $level_count = $this->_level_count[$sub] = 0;
    $parent_tree = $this->_parent_tree[$sub][] = "";


    $parent_tree[$level_count] = $sub_env;
    $level_count++;

    $inherited_params = array('Read_Access_ID', 'Write_Access_ID', 'Edit_Access_ID','Checked_Access_ID',
                                'Delete_Access_ID', 'Subscribe_Access_ID', 'Moderation_ID' );
    if ( $nc_core->modules->get_by_keyword("cache") ) {
      $inherited_params[] = 'Cache_Access_ID';
      $inherited_params[] = 'Cache_Lifetime';
    }

    $parent_sub = $sub_env["Parent_Sub_ID"];
    
    while ($parent_sub) {
      $parent_sub_env = $this->get_by_id($parent_sub);

      if (!$sub_env["Template_ID"]) {
        $sub_env["Template_ID"] = $parent_sub_env["Template_ID"];
        $sub_env["TemplateSettings"] = $parent_sub_env["TemplateSettings"];
      }
      

      foreach ( $inherited_params as $v ) {
        if (!$sub_env[$v]) $sub_env[$v] = $parent_sub_env[$v];
      }


      $parent_tree[$level_count] = $parent_sub_env;
      $sub_env = $this->inherit_system_fields($this->essence, $parent_tree[$level_count], $sub_env);

      $level_count++;

      $parent_sub = $parent_sub_env["Parent_Sub_ID"];
    }

    $catalogue_env = $nc_core->catalogue->get_by_id($sub_env["Catalogue_ID"]);

    $sub_env = $this->inherit_system_fields("Catalogue", $catalogue_env, $sub_env);

    if (!$sub_env["Template_ID"]) {
     $sub_env["Template_ID"] = $catalogue_env["Template_ID"];
     $sub_env["TemplateSettings"] = $catalogue_env["TemplateSettings"];
    }

    foreach ( $inherited_params as $v ) {
      if (!$sub_env[$v]) $sub_env[$v] = $catalogue_env[$v];
    }

    $parent_tree[$level_count] = $catalogue_env;
    $parent_tree[$level_count]["Subdivision_Name"] = $catalogue_env["Catalogue_Name"];
    $parent_tree[$level_count]["Hidden_URL"] = "/";
    
    // store data
    $this->_level_count[$sub] = $level_count;
    $this->_parent_tree[$sub] = $parent_tree;
    
    return $sub_env;
  }
  
  public function get_parent_tree ($sub) {
    // get data
    $this->get_by_id($sub);
    // check
    if ( is_array($this->_parent_tree[$sub]) && !empty($this->_parent_tree[$sub]) ) {
      return $this->_parent_tree[$sub];
    }
    // parent sub array not found
    return false;
  }
  
  public function get_level_count ($sub) {
    // get data
    $this->get_by_id($sub);
    // check
    if ( isset($this->_level_count[$sub]) ) {
      return $this->_level_count[$sub];
    }
    // level count array not found
    return false;
  }


  public function validate_english_name ($str) {
    // validate Hidden_URL
    return nc_preg_match("/^[a-zа-я0-9-]+$/i", $str);
  }
  
}


?>
