<?php

abstract class nc_Essence extends nc_System {
  
  protected $current;
  protected $system_tables, $inherit_essences, $essence;
  protected $db;
  protected $data;
  
  protected function __construct () {
    
    // system tables
    $this->system_tables = array("Catalogue", "Subdivision", "User", "Template");
    // inherit essences
    $this->inherit_essences = array("Subdivision", "Sub_Class", "Template");
    // set essence
    $this->essence = str_replace( "nc_", "", get_class($this) );
    
  }
  
  public function get_current ($item = "") {

    if ( empty($this->current) ) return false;
    
    if ( $item ) {
      return array_key_exists($item, $this->current) ? $this->current[$item] : "";
    }
    else {
      return $this->current;
    }
    
  }
  
  public function set_current_by_id ($id, $reset = false) {
    // validate
    $id = intval($id);
    
    if ($id) {
      $this->current = $this->get_by_id($id, "", $reset);
      // return result
      return $this->current;
    }
    
    // reject
    return false;    
  }
  
  public function set_current_item ($item, $value) {
  
    if ( empty($this->current) ) return false;
    
    if ( array_key_exists($item, $this->current) ) {
      $this->current[$item] = $value;
    }
    
    return true;  
  }
  
  public function get_by_id ($id, $item = "", $reset = false) {
    // call as static
    static $storage = array();
 
    // validate parameters
    $id = intval($id);
   
    // check inited object
    if ( empty($storage[$id]) || $reset ) {
      $storage[$id] = $this->db->get_row("SELECT * FROM `".$this->essence."`
      WHERE `".$this->essence."_ID` = '".$id."'", ARRAY_A);
      
      // convert system fields data
      if ( in_array($this->essence, $this->system_tables) ) { 
        $storage[$id] = $this->convert_system_vars($storage[$id]);
      }
      
      // inherit current data
      if ( in_array($this->essence, $this->inherit_essences) ) {
        $storage[$id] = $this->inherit($storage[$id]);
      }
    }
   
    // if item requested return item value
    if ( $item && is_array($storage[$id]) ) {
      return array_key_exists($item, $storage[$id]) ? $storage[$id][$item] : "";
    }
    
    return $storage[$id];
  }
  
  public function delete_by_id ($id) {
    // validate parameters
    $id = intval($id);
    
    $this->db->query("DELETE FROM `".$this->essence."` WHERE `".$this->essence."_ID` = '".$id."'");
    
    return $this->db->rows_affected;
  }
  
  public function convert_system_vars ($env_array, $reset = 0) {
    // call as static
    static $storage = array();
    
    if ( !empty($storage[$this->essence][ $env_array[$this->essence."_ID"] ]) && !$reset) {
      return $storage[$this->essence][ $env_array[$this->essence."_ID"] ];
    }
    
    /* $debug = debug_backtrace();
    $debug_arr = array();
    foreach ($debug as $value) {
      $debug_arr[] = $value['function'];
    }
    echo join( " -> ",  array_reverse($debug_arr) )."<br/>";
    echo $this->essence." - ".$env_array[$this->essence."_ID"]."<br/><br/>"; */
    
    // system superior object
    $nc_core = nc_Core::get_object();
  
    // load system table fields
    $table_fields = $nc_core->get_system_table_fields($this->essence);
    // count
    $counted_fileds = count( $table_fields ); 
    // поля типа файл
    $file_field = array();
    $filetable = array ();
    // найдем все поля типа файл
    for ( $i = 0; $i < $counted_fileds; $i++ ) {
      if ( $table_fields[$i]['type'] == 6 ) {
        $file_field[$table_fields[$i]['id']] = $table_fields[$i]['id'];
      }
    }
    
    // если есть поля типа файл, то выполним запрос к Filetable
    if ( !empty($file_field) ) {
      $file_in_table = $this->db->get_results("SELECT `Virt_Name`, `File_Path`, `Message_ID`, `Field_ID` 
        FROM `Filetable` 
        WHERE `Field_ID` IN (".join(',', $file_field).")", ARRAY_A);
      if ( !empty($file_in_table) ) {
        foreach ( $file_in_table as $v ) {
          $filetable[$v['Message_ID']][$v['Field_ID']] = array($v['Virt_Name'], $v['File_Path']);
        }
      }
    }
    
    // Проход по всем полям
    for ($i=0; $i < $counted_fileds; $i++) {
      $field_id = $table_fields[$i]['id'];
      $field_name = $table_fields[$i]['name'];
      $field_type = $table_fields[$i]['type'];
      $field_format = $this->db->escape( $table_fields[$i]['format'] );
      $field_inherit = $table_fields[$i]['inheritance'];

      if ($env_array[$field_name]) {
        switch ($field_type) {
          // Select
          case 4:
            $listname = $this->db->get_var("SELECT `".$field_format."_Name` FROM `Classificator_".$field_format."` WHERE `".$field_format."_ID` = '".$env_array[$field_name]."'");

            $env_array[$field_name."_id"] = $env_array[$field_name];
            $env_array[$field_name] = $listname;
          break;
          // File
          case 6:
            //file_data - массив с ориг.названием, типом, размером, [именем_файла_на_диске]
            $file_data = explode(':', $env_array[$field_name]);

            $env_array[$field_name."_name"] = $file_data[0]; // оригинальное имя
            $env_array[$field_name."_type"] = $file_data[1]; // тип
            $env_array[$field_name."_size"] = $file_data[2]; // размер
            $ext = substr( $file_data[0], strrpos($file_data[0], ".") );  // расширение

            // запись в таблице Filetable
            $row = $filetable[$env_array[$this->essence."_ID"]][$field_id];
            if ($row) {
              // Proteced FileSystem
              $env_array[$field_name] = $nc_core->get_variable("SUB_FOLDER").$nc_core->get_variable("HTTP_FILES_PATH").ltrim($row[1], '/')."h_".$row[0];
              $env_array[$field_name."_url"] = $nc_core->get_variable("SUB_FOLDER").$nc_core->get_variable("HTTP_FILES_PATH").ltrim($row[1], '/').$row[0];  
            }
            else {
              if ($file_data[3]) {
                // Original FileSystem
                $env_array[$field_name] = $env_array[$field_name."_url"] = $nc_core->get_variable("SUB_FOLDER").$nc_core->get_variable("HTTP_FILES_PATH").$file_data[3];   
              }
              else {
                // Simple FileSysytem
                $env_array[$field_name] = $env_array[$field_name."_url"] = $nc_core->get_variable("SUB_FOLDER").$nc_core->get_variable("HTTP_FILES_PATH").$field_id."_".$env_array[$this->essence."_ID"].$ext;  
              }
            }

          break;
          case 8:
            $env_array[$field_name."_year"] = substr($env_array[$field_name], 0, 4);
            $env_array[$field_name."_month"] = substr($env_array[$field_name], 5, 2);
            $env_array[$field_name."_day"] = substr($env_array[$field_name], 8, 2);
            $env_array[$field_name."_hours"] = substr($env_array[$field_name], 11, 2);
            $env_array[$field_name."_minutes"] = substr($env_array[$field_name], 14, 2);
            $env_array[$field_name."_seconds"] = substr($env_array[$field_name], 17, 2);
          break;
          // Multiselect
          case 10:
            $array_with_id = explode(',' , $env_array[$field_name] );

            if (!$array_with_id[0]) unset($array_with_id[0]); 
            if (!$array_with_id[count($array_with_id)]) unset($array_with_id[count($array_with_id)]);
            if (empty($array_with_id)) break;
            // латинское имя списка
            $table_name = strtok($field_format, ':'); 
            // получим сами элементы
            $listname = $this->db->get_col("SELECT `".$table_name."_Name` 
              FROM `Classificator_".$table_name."`
              WHERE `".$table_name."_ID` IN(".join(',', $array_with_id).")");
            $env_array[$field_name."_id"] = (array)$array_with_id;
            $env_array[$field_name] = (array)$listname;
          break;
        }
      }
    }

    $storage[$this->essence][ $env_array[$this->essence."_ID"] ] = $env_array;
    
    return $env_array;
  }
  
  
  public function inherit_system_fields ($system_table_name, $parent_array, $child_array) {
    // system superior object
    $nc_core = nc_Core::get_object();
    
    // load system table fields
    $table_fields = $nc_core->get_system_table_fields($system_table_name);
    // count
    $counted_fileds = count( $table_fields );

    for ($i=0; $i < $counted_fileds; $i++) {
      // не наследуется
      if (!$table_fields[$i]['inheritance']) continue;
      // field name
      $field_name = $table_fields[$i]['name'];

      if ( !array_key_exists($field_name, $child_array)  || $child_array[$field_name]=="" ) {
        switch ($table_fields[$i]['type']) {
          // list
          case 4:
            $child_array[$field_name] = $parent_array[$field_name];
            $child_array[$field_name.'_id'] = $parent_array[$field_name.'_id'];
          break;
          // file
          case 6:
            $child_array[$field_name] = isset($parent_array[$field_name]) ? $parent_array[$field_name] : "";
            $child_array[$field_name.'_name'] = isset($parent_array[$field_name.'_name']) ? $parent_array[$field_name] : "";
            $child_array[$field_name.'_size'] = isset($parent_array[$field_name.'_size']) ? $parent_array[$field_name] : "";
            $child_array[$field_name.'_type'] = isset($parent_array[$field_name.'_type']) ? $parent_array[$field_name] : "";
            $child_array[$field_name.'_url'] = isset($parent_array[$field_name.'_url']) ? $parent_array[$field_name] : "";
          break;
          //date
          case 8:
            $child_array[$field_name] = $parent_array[$field_name];
            $child_array[$field_name.'_day'] = $parent_array[$field_name.'_day'];
            $child_array[$field_name.'_month'] = $parent_array[$field_name.'_month'];
            $child_array[$field_name.'_year'] = $parent_array[$field_name.'_year'];
            $child_array[$field_name.'_hours'] = $parent_array[$field_name.'_hours'];
            $child_array[$field_name.'_minutes'] = $parent_array[$field_name.'_minutes'];
            $child_array[$field_name.'_seconds'] = $parent_array[$field_name.'_seconds'];
          break;
          //multilist
          case 10:
            $child_array[$field_name] = $parent_array[$field_name];
            $child_array[$field_name.'_id'] = $parent_array[$field_name.'_id'];   
          break;
          // other fields
          default:
            $child_array[$field_name] = $parent_array[$field_name];
        }
      }
      // список наследуется - если (элемент == 0)
      else if ($table_fields[$i]['type']==4 && $child_array[$field_name.'_id']==0) {
        $child_array[$field_name] = $parent_array[$field_name];
        $child_array[$field_name.'_id'] = $parent_array[$field_name.'_id'];
      }
    }

    return $child_array;
  }
  
}

?>